


//  Start of int_array implementation

/*
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Version         v1   14 May 2025
  Compatibility   TSE v4 upwards, all variants

  Only use the following functions to manipulate integer arrays:
    integer proc int_array_create(string array_name)
    integer proc int_array_delete(string array_name)
    integer proc int_array_set(string array_name, integer index, integer value)
    integer proc int_array_get(string array_name, integer index)

  int_array_get() returns the value at the index, which defaults to 0.
  All other functions return TRUE or FALSE if they went ok or not.

  An array is intended to only be used in the macro it is created in.
  An array name only needs to be unique within the macro it is created in.
  You can create as many arrays as TSE has free memory and bufferids.
  Creating an array will use up a bufffer id, which limits you to MAXINT
  arrays minus how many bufferids other TSE programs use.
  Array elements can have an index from MININT to MAXINT.
  When an array is created, all array elements exist with value 0.
  Array elements with value 0 do not use memory.
  Setting an array element from not 0 to 0 frees up its memory usage.
  Deleting an array will free up all its memory usage at once.


  NOTE
    I maintain int_array's source as a separate include-file,
    and as an independent and versioned source of truth.

    But for actual use I copy int_array's source as a whole into each macro
    that needs it, so that each such a macro can be tested and distributed
    without int_arrays's version being an outside dependency.

*/


// Do not use or call these int_array_internal_... variables and functions!

integer int_array_internal_id                    = 0
string  int_array_internal_prefix [MAXSTRINGLEN] = 'IntArray'


proc int_array_internal_context()
  integer org_id = 0

  if int_array_internal_id == 0
    int_array_internal_prefix = SplitPath(CurrMacroFilename(), _NAME_) + ':IntArray'
    org_id                    = GetBufferId()
    int_array_internal_id     = CreateTempBuffer()
    ChangeCurrFilename(int_array_internal_prefix, _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_)
    int_array_internal_prefix = int_array_internal_prefix + ':'
    GotoBufferId(org_id)
  endif
end int_array_internal_context


proc int_array_internal_warn(string array_name, string msg)
  if (Query(MsgLevel) in _ALL_MESSAGES_, _WARNINGS_ONLY_)
    if Query(Beep)
      Alarm()
    endif

    MsgBox(int_array_internal_prefix + array_name,
           iif(Val(msg),
               'ERROR ' + msg + '.',
               'ERROR: ' + msg))
  endif
end int_array_internal_warn

// End of int_array_internal_... variables and functions.


// Start of externally callable int_array functions:

integer proc int_array_create(string array_name)
  integer array_id = 0
  integer ok       = FALSE
  integer org_id   = GetBufferId()

  int_array_internal_context()

  array_id = GetBufferInt(int_array_internal_prefix + array_name,
                          int_array_internal_id)
  if array_id
    int_array_internal_warn(array_name, '1')
  else
    array_id = CreateTempBuffer()

    if array_id
      ChangeCurrFilename(int_array_internal_prefix + array_name,
                         _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_)
      if SetBufferInt(int_array_internal_prefix + array_name,
                      array_id,
                      int_array_internal_id)
        ok = TRUE
      else
        int_array_internal_warn(array_name, '2')
      endif
    else
      int_array_internal_warn(array_name, '3')
    endif

    GotoBufferId(org_id)
  endif

  return(ok)
end int_array_create


integer proc int_array_delete(string array_name)
  integer array_id = 0
  integer ok       = FALSE

  int_array_internal_context()
  array_id = GetBufferInt(int_array_internal_prefix + array_name,
                          int_array_internal_id)
  if array_id
    ok = TRUE

    if not AbandonFile(array_id)
      int_array_internal_warn(array_name, '4')
      ok = FALSE
    endif

    if not DelBufferVar(int_array_internal_prefix + array_name,
                        int_array_internal_id)
      int_array_internal_warn(array_name, '5')
      ok = FALSE
    endif
  else
    int_array_internal_warn(array_name, '6')
  endif

  return(ok)
end int_array_delete


integer proc int_array_set(string array_name, integer index, integer value)
  integer array_id = 0
  integer ok       = FALSE

  int_array_internal_context()
  array_id = GetBufferInt(int_array_internal_prefix + array_name,
                          int_array_internal_id)

  if array_id
    if value
      if SetBufferInt(int_array_internal_prefix + array_name + Str(index),
                      value,
                      array_id)
        ok = TRUE
      else
        int_array_internal_warn(array_name, '7')
      endif
    else
      DelBufferVar(int_array_internal_prefix + array_name + Str(index),
                   array_id)
    endif
  else
    int_array_internal_warn(array_name, '8')
  endif

  return(ok)
end int_array_set


integer proc int_array_get(string array_name, integer index)
  integer array_id = 0
  integer value    = 0

  int_array_internal_context()
  array_id = GetBufferInt(int_array_internal_prefix + array_name,
                          int_array_internal_id)

  if array_id
    value = GetBufferInt(int_array_internal_prefix + array_name + Str(index),
                         array_id)
  endif

  return(value)
end int_array_get

// End of int_array implementation



